
#include "sc_function.h"

SoftwareSerial mySerial(sc_rx, sc_tx); // RX, TX

bool compare_ret(String order,String ret_compare) {
  String temp_ret = run_order(order);
  if(temp_ret.indexOf(ret_compare) == -1) {
    return false;
  }
  return true;
}

String run_order(String order) {
  String send_str = "";
  String receive_str = "";

  send_str = create_order(order);
  Serial.print(order);
  Serial.print("?");
  Serial.print(send_str);
  Serial.print("?");

  mySerial.print(send_str);
  receive_str = mySerial.readStringUntil('.');
  Serial.print(send_str + receive_str);
  return receive_str;
}

String create_order(String order_str) {
  int len = order_str.length();
  char parity0 = 0;
  char parity1 = 0;
  for (int i = 0;i < len;i += 2) {
    parity0 ^= change_hex(order_str.charAt(i));
    parity1 ^= change_hex(order_str.charAt(i+1));
  }
  String ret_str = "\n";
  ret_str += order_str;
  ret_str += String(parity0,HEX);
  ret_str += String(parity1,HEX);
  ret_str += ":";
  ret_str.toUpperCase();
  return ret_str;  
}

char change_hex(char a) {
  if (a >= 0x30 and a <= 0x39)  {
     //0 to 9
     return (a -0x30);
  }
  else  {
     //A to F
     return (a - 0x41 + 10);
  }
}

void sc_idle_packet() {
  static boolean packet_toggle = true;
  //Serial.print("idle");
  if(packet_toggle) {
    run_order("010503");
  }
  else {
    run_order("010400");
  }
  packet_toggle = !packet_toggle;
}

void sc_init_serial() {
  mySerial.begin(38400); // ソフトウェアシリアルの初期化
  mySerial.setTimeout(300);  
}

bool sc_SetLocoFunction(word address, unsigned char inFunction, unsigned char inPower) {
  //underconstruction
  //Locoが３以外なら受け付けない！
  //if(address - ADDR_DCC != 3)
  //  return true;
  switch(inFunction) {
    case 1:
      //警笛１
      if(inPower == 1){run_order("01018400");delay(100);run_order("01018000010001"); }
      else {run_order("01018000010000");}
      break;
    case 2:
      //ドア　排水
      if(inPower == 1){run_order("01018000020002");}
      else {run_order("01018000020000");}
      break;
    case 3:
      //制動　
      if(inPower == 1){run_order("01018000040004");}
      else {run_order("01018000040000");}
      break;
    case 4:
      //C-ATS 注水
      if(inPower == 1){run_order("01018000080008");}
      else {run_order("01018000080000");}
      break;
    case 5:
      //惰行　絶気
      if(inPower == 1){run_order("01018000100010");}
      else {run_order("01018000100000");}
      break;
    case 6:
      //ブザー　投炭
      if(inPower == 1){run_order("01018000200020");}
      else {run_order("01018000200000");}
      break;
    case 7:
      //警笛２
      if(inPower == 1){run_order("01018401");delay(100);run_order("01018000010001"); }
      else {run_order("01018000010000");}
      break;
    case 8:
      //警笛３
      if(inPower == 1){run_order("01018402");delay(100);run_order("01018000010001"); }
      else {run_order("01018000010000");}
      break;
    default:
      break;
  }
  return true;
}

boolean sc_SetLocoDirection(word address, unsigned char inDirection) {
  //underconstruction
  //Locoが３以外なら受け付けない！
  //if(address - ADDR_DCC != 3)
  //  return true;
  if(inDirection == 1)
    run_order("010181010000");
  else
    run_order("010181010001");
  return true;
}

boolean sc_SetLocoSpeedEx(word address, int inSpeed, int inProtcol) {
  //Locoが３以外なら受け付けない！
  //if(address - ADDR_DCC != 3)
  //  return true;
  //speed (0 - 1023(0x3FF)) ->(0 - 4095(0xFFF)) -> *4 OK 
  String order_str = "010181020";
  int temp_speed = inSpeed * 4;
  int speed0 = (temp_speed / 256) & 0xf;
  int speed1 = (temp_speed / 16 ) & 0xf;
  int speed2 = (temp_speed % 16 ) & 0xf;
  order_str += String(speed0,HEX);
  order_str += String(speed1,HEX);
  order_str += String(speed2,HEX);
  order_str.toUpperCase();
  run_order(order_str);
  return true;
}
